import { Injectable, Logger } from '@nestjs/common';
import { SchedulerRegistry } from '@nestjs/schedule';
import { CronJob } from 'cron';
import { SystemUsageService } from './system-usage.service';
import * as si from 'systeminformation';
import * as child_process from 'child_process';
import * as xml2js from 'xml2js';
import * as _ from 'loadsh';

@Injectable()
export class SystemUsageCron {
  private readonly logger = new Logger(SystemUsageCron.name);

  constructor(
    private readonly schedulerRegistry: SchedulerRegistry,
    private readonly systemUsageService: SystemUsageService,
  ) {}
  async handleGpuInfo(): Promise<{ usage: number }> {
    const { stdout, stderr } = await new Promise<{
      stdout: string;
      stderr: string;
    }>((res, rej) => {
      child_process.exec(
        process.env.GPU_MODEL === 'P800' ? 'xpu-smi -x -q' : 'nvidia-smi -x -q',
        (error, stdout, stderr) => {
          if (error) {
            rej(error);
          } else if (stderr) {
            console.log('stderr', stderr);
          } else {
            res({ stdout, stderr });
          }
        },
      );
    });
    if (stderr) {
      this.logger.error('执行脚本时发生错误:', stderr);
      return { usage: 0 };
    }
    // 解析XML数据
    interface XPUUtilization {
      xpu_util: string[];
    }
    interface GPUUtilization {
      gpu_util: string[];
    }

    interface XPUItem {
      $: { id: string };
      product_name: string[];
      utilization: XPUUtilization[];
    }
    interface GPUItem {
      $: { id: string };
      product_name: string[];
      utilization: GPUUtilization[];
    }

    interface XPUSmiLog {
      xpu_smi_log: {
        xpu: XPUItem[];
      };
    }
    interface NvidiaSmiLog {
      nvidia_smi_log: {
        gpu: GPUItem[];
      };
    }
    this.logger.log('stdout', stdout)
    return new Promise((resolve, reject) => {
      xml2js.parseString(
        stdout,
        (err: Error | null, result: XPUSmiLog | NvidiaSmiLog) => {
          if (err) {
            this.logger.error('解析XML数据时发生错误:', err.message);
            reject(err);
            return;
          }

          // 处理解析后的数据
          if (process.env.GPU_MODEL === 'P800') {
            const xpuData = (result as XPUSmiLog).xpu_smi_log.xpu;
            this.logger.log('以下时xpu打他');
            this.logger.log(xpuData);
            this.logger.log(_.get(xpuData, '[0].utilization[0].xpu_util[0]'));
            resolve({
              usage: parseFloat(
                _.get(xpuData, '[0].utilization[0].xpu_util[0]'),
              ),
            });
          } else {
            const cpuData = (result as NvidiaSmiLog).nvidia_smi_log.gpu;
            this.logger.log('以下时cpu打他');
            this.logger.log(_.get(cpuData, '[0]'));
            resolve({
              usage: parseFloat(
                _.get(cpuData, '[0].utilization[0].gpu_util[0]'),
              ),
            });
          }
        },
      );
    });
  }

  /**
   * 启动定时任务，用于定期记录系统使用情况
   */
  startCronJob() {
    // 每分钟执行一次
    const job = new CronJob('* * * * *', async () => {
      try {
        const [cpu, memory, gpuInfo] = await Promise.all([
          si.currentLoad(),
          si.mem(),
          this.handleGpuInfo(),
        ]);
        this.logger.log('gpu', gpuInfo.usage);
        this.logger.log('cpu', parseFloat(cpu?.currentLoad.toFixed(2)));
        this.logger.log('memory', memory)
        this.logger.log('memory used', parseFloat(memory?.active.toFixed(2)))
        this.logger.log('memory total', parseFloat(memory?.total.toFixed(2)))

        await this.systemUsageService.create({
          cpuUsage: parseFloat(cpu?.currentLoad.toFixed(2)),
          gpuUsage: parseFloat(gpuInfo.usage.toFixed(2)),
          memoryUsage: parseFloat((memory.active / memory.total*100).toFixed(2)),
        });

        this.logger.log(
          `系统指标已记录: CPU ${cpu.currentLoad.toFixed(2)}%, 内存 ${((memory.active / memory.total) * 100).toFixed(2)}% gpu ${gpuInfo.usage}`,
        );
      } catch (error: unknown) {
        if (error instanceof Error) {
          this.logger.error('记录系统指标失败', error.message);
        } else {
          this.logger.error('记录系统指标失败', String(error));
        }
      }
    });

    this.schedulerRegistry.addCronJob('systemUsageJob', job);
    job.start();
    this.logger.log('系统指标定时任务已启动');
  }
}
